/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.plugins.findbugs;

import ch.hortis.sonar.model.*;
import static org.apache.commons.collections.CollectionUtils.*;
import org.sonar.plugins.api.Languages;
import org.sonar.plugins.api.maven.*;
import org.sonar.plugins.api.rules.RulesManager;
import org.w3c.dom.Element;

import java.util.List;

class FindbugsFailureCollector extends AbstractFailureCollector {
  private RulesProfile activeProfile;

  FindbugsFailureCollector(JavaMeasuresRecorder measuresRecorder, RulesManager rulesManager) {
    super(measuresRecorder, rulesManager);
    activeProfile = rulesManager.getActiveProfile(Languages.JAVA);
  }

  @Override
  protected String xpathForFiles() {
    return "/BugCollection/file";
  }

  @Override
  protected String elementNameForFailures() {
    return "BugInstance";
  }

  @Override
  protected String javaFilePath(Element element) {
    return classnameToFilepath(element.getAttribute("classname"), sourceRoots);
  }

  private static String classnameToFilepath(String filename, List<String> sourceDirs) {
    String srcDir = "";
    if (isNotEmpty(sourceDirs)) {
      srcDir = sourceDirs.get(0) + '/';
    }
    return srcDir + filename.replace('.', '/') + ".java";
  }

  @Override
  protected String ruleKey(Element failure) {
    return failure.getAttribute("type");
  }

  @Override
  protected String keyForPlugin() {
    return FindbugsPlugin.KEY;
  }

  @Override
  protected RuleFailureLevel getLevel(Element error) {
    String ruleKey = ruleKey(error);
    ActiveRule activeRule = activeProfile.getActiveRule(keyForPlugin(), ruleKey);
    if (activeRule == null) {
      throw new IllegalStateException("Active rule '" + ruleKey + "' could not be found when setting level.");
    }
    return activeRule.getLevel();
  }

  @Override
  protected String lineNumberFor(Element failure) {
    return failure.getAttribute("lineNumber");
  }

  @Override
  protected String messageFor(Element error) {
    return error.getAttribute("message");
  }
}
