/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.plugins.findbugs;

import ch.hortis.sonar.model.*;
import org.sonar.plugins.api.Languages;
import org.sonar.plugins.api.maven.*;
import org.sonar.plugins.api.maven.model.MavenPom;
import org.sonar.plugins.api.rules.RulesManager;
import org.apache.maven.plugin.logging.Log;

import java.io.File;
import java.util.List;

public class FindbugsMavenCollector extends AbstractJavaMavenCollector {
  private final AbstractFailureCollector collector;
  private final RulesManager rulesManager;

  public FindbugsMavenCollector(JavaMeasuresRecorder measuresRecorder, Log log, RulesManager rulesManager) {
    this(measuresRecorder, log, rulesManager, new FindbugsFailureCollector(measuresRecorder, rulesManager));
  }

  public FindbugsMavenCollector(JavaMeasuresRecorder measuresRecorder, Log log, RulesManager rulesManager, AbstractFailureCollector collector) {
    super(measuresRecorder, log);
    this.rulesManager = rulesManager;
    this.collector = collector;
  }

  public Class<? extends MavenPluginHandler> dependsOnMavenPlugin(MavenPom pom) {
    return FindbugsMavenPluginHandler.class;
  }

  @Override
  public boolean shouldCollectOn(MavenPom pom) {
    if (super.shouldCollectOn(pom)) {
      RulesProfile rulesProfile = rulesManager.getActiveProfile(Languages.JAVA);
      List<ActiveRule> activeRules = rulesProfile.getActiveRulesByPlugin(FindbugsPlugin.KEY);
      return !activeRules.isEmpty();
    }
    return false;
  }

  public boolean shouldStopOnFailure() {
    return true;
  }

  @Override
  protected boolean shouldCollectIfNoSources() {
    return false;
  }

  public void collect(MavenPom pom) {
    List<String> sourceDirs = pom.getCompileSourceRoots();
    File reportFile = MavenCollectorUtils.findFileFromBuildDirectory(pom, "findbugs.xml");

    collector.collect(reportFile, sourceDirs);
  }
}