/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.plugins.findbugs;

import org.apache.commons.io.FileUtils;
import org.sonar.plugins.api.Languages;
import org.sonar.plugins.api.maven.*;
import org.sonar.plugins.api.maven.model.*;
import org.sonar.plugins.api.rules.RulesManager;

import java.io.*;

public class FindbugsMavenPluginHandler extends AbstractMavenPluginHandler {
  private RulesManager rulesManager;
  private FindbugsRulesRepository findbugsRulesRepository;

  public FindbugsMavenPluginHandler(RulesManager rulesManager, FindbugsRulesRepository findbugsRulesRepository) {
    this.rulesManager = rulesManager;
    this.findbugsRulesRepository = findbugsRulesRepository;
  }

  public String getGroupId() {
    return MavenUtils.CODEHAUS_MOJO_GROUP_ID;
  }

  public String getArtifactId() {
    return "findbugs-maven-plugin";
  }

  public String getVersion() {
    return "1.2";
  }

  public boolean isFixedVersion() {
    return false;
  }

  public boolean shouldStopOnFailure() {
    return true;
  }

  public String[] getGoals() {
    return new String[]{"findbugs"};
  }

  @Override
  public void configurePlugin(MavenPom pom, MavenPlugin plugin) {
    configureClassesDir(pom, plugin);

    plugin.setConfigParameter("xmlOutput", "true");
    plugin.setConfigParameter("threshold", "Low");
    plugin.setConfigParameter("effort", "Max");
    plugin.addDependency("net.sourceforge.findbugs", "findbugs", "1.3.2", "jar");

    try {
      File xmlFile = saveConfigXml(pom.getSonarWorkingDirectory());
      plugin.setConfigParameter("includeFilterFile", xmlFile.getCanonicalPath());
    }
    catch (IOException e) {
      throw new RuntimeException("Failed to save the findbugs XML configuration.", e);
    }
  }

  private void configureClassesDir(MavenPom pom, MavenPlugin plugin) {
    File classesDir = pom.getBuildOutputDir();
    if (classesDir==null || !classesDir.exists()) {
      throw new RuntimeException("Findbugs needs sources to be compiled. " +
          "Please edit pom.xml to set the <ouputDirectory> node and build before executing sonar."
      );
    }
    try {
      plugin.setConfigParameter("classFilesDirectory", classesDir.getCanonicalPath());
    } catch (Exception e) {
      throw new RuntimeException("Invalid classes directory", e);
    }
  }

  private File saveConfigXml(File workingDirectory) throws IOException {
    String configuration = findbugsRulesRepository.exportConfiguration(rulesManager.getActiveProfile(Languages.JAVA));
    File findbugsIncludeFile = new File(workingDirectory, "findbugs-include.xml");
    FileUtils.writeStringToFile(findbugsIncludeFile, configuration);
    return findbugsIncludeFile;
  }
}
